@@ -32,7 +32,7 @@ use Error qw(:try);             # for web services verification (you may comment
 use Data::Dumper;               # dump hashes for debugging
 use File::Temp qw/ tempfile /;  # create temp files
 
-our $VERSION = '1.84';
+our $VERSION = '1.88';
 
 =head1 NAME
 
@@ -92,6 +92,12 @@ Set maximum number of HTTP redirects. Default is 0.
 
 Sets a proxy which is then used for http and https requests.
 
+ ex.: http://proxy.company.net:3128
+
+with authentication:
+
+ ex.: http://user:password@proxy.company.net:3128
+
 =item output_dir
 
 Output directory where all logfiles will go to. Defaults to current directory.
@@ -189,8 +195,6 @@ sub engine {
 
     $self->_processcasefile();
 
-    my $useragent = $self->_get_useragent();
-
     # write opening tags for STDOUT.
     $self->_writeinitialstdout();
 
@@ -243,7 +247,7 @@ sub engine {
             $repeat = $xmltestcases->{repeat};
         }
 
-        my $useragent = $self->_get_useragent();
+        my $useragent = $self->_get_useragent($xmltestcases->{case});
 
         for my $run_nr (1 .. $repeat) {
 
@@ -298,9 +302,6 @@ sub engine {
     my $endruntimer = time();
     $self->{'result'}->{'totalruntime'} = ( int( 1000 * ( $endruntimer - $startruntimer ) ) / 1000 );    #elapsed time rounded to thousandths
 
-    # required to clean up cookie files
-    undef $useragent;
-
     # do return/cleanup tasks
     return $self->_finaltasks();
 }
@@ -320,7 +321,7 @@ sub _run_test_case {
     $case->{'iscritical'}   = 0;
     $case->{'messages'}     = [];
 
-    $useragent = $self->_get_useragent() unless defined $useragent;
+    $useragent = $self->_get_useragent({1 => $case}) unless defined $useragent;
 
     # don't do this if monitor is disabled in gui
     if($self->{'gui'} and $self->{'monitorenabledchkbx'} ne 'monitor_off') {
@@ -342,7 +343,7 @@ sub _run_test_case {
         my($uri)=URI->new($case->{url});
         my($host)=$uri->host();
         $useragent->default_header('Host' => $uri->host());
-        $case->{url}=~s/$host/$self->{'config'}->{'realserverip'}/;
+        $case->{url}=~s/\Q$host\E/$self->{'config'}->{'realserverip'}/mx;
     }
 
     if( $self->{'gui'} ) { $self->_gui_tc_descript($case); }
@@ -580,13 +581,22 @@ sub _run_test_case {
 
 ################################################################################
 sub _get_useragent {
-    my $self = shift;
+    my($self, $testcases) = @_;
 
-    # construct LWP object
-    my $useragent  = LWP::UserAgent->new(keep_alive=>1);
+    # keepalive is required for ntml authentication but breaks
+    # https proxy support, so try determince which one we need
+    my $keepalive = 1;
+    if($testcases and $self->{'config'}->{'proxy'}) {
+        for my $nr (keys %{$testcases}) {
+            if($testcases->{$nr}->{'url'} =~ m/^https/gmx) {
+                $keepalive = 0;
+            }
+        }
+    }
+    my $useragent  = LWP::UserAgent->new(keep_alive=>$keepalive);
 
     # store cookies in our LWP object
-    my($fh, $cookietempfilename) = tempfile(undef, UNLINK => 1);
+    my($fh, $cookietempfilename) = tempfile();
     unlink ($cookietempfilename);
     $useragent->cookie_jar(HTTP::Cookies->new(
                                                  file     => $cookietempfilename,
@@ -603,10 +613,40 @@ sub _get_useragent {
 
     # add proxy support if it is set in config.xml
     if( $self->{'config'}->{'proxy'} ) {
+        # try IO::Socket::SSL first
+        eval {
+            require IO::Socket::SSL;
+            IO::Socket::SSL->import();
+        };
         my $proxy = $self->{'config'}->{'proxy'};
-        $proxy    =~ s/^http:\/\///mx;
-        $useragent->proxy([qw( http )], "http://".$proxy);
-        $ENV{'HTTPS_PROXY'} = "http://".$proxy;
+        $proxy    =~ s/^http(s|):\/\///mx;
+        # http just works
+        $useragent->proxy('http', 'http://'.$proxy);
+        # authentication?
+        my $proxyuser = '';
+        my $proxypass = '';
+        if($proxy =~ s/^(.*?):(.*?)@(.*)$/$3/gmx) {
+            $proxyuser = $1;
+            $proxypass = $2;
+        }
+        # ssl depends on which class we have
+        if($INC{'IO/Socket/SSL.pm'}) {
+            $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS} = "IO::Socket::SSL";
+            if($proxypass) {
+                $proxy = $proxyuser.':'.$proxypass.'@'.$proxy;
+            }
+            my $con_proxy = 'connect://'.$proxy;
+            $useragent->proxy('https', $con_proxy);
+        } else {
+            # ssl proxy only works this way, see http://community.activestate.com/forum-topic/lwp-https-requests-proxy
+            $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS}   = "Net::SSL";
+            $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME}      = 0;
+            $ENV{HTTPS_PROXY}                       = $proxy;
+            $ENV{HTTPS_PROXY_USERNAME}              = $proxyuser;
+            $ENV{HTTPS_PROXY_PASSWORD}              = $proxypass;
+            # env proxy breaks the ssl proxy above
+            #$useragent->env_proxy();
+        }
     }
 
     # don't follow redirects unless set by config
